#! /bin/sh
#
#   startup.sh -- スタートアップスクリプト (floppy版)
#
#	0.0: Jun. 19, 2004 by Dai ISHIJIMA
#	0.1: Feb. 17, 2007 (乱数パターンと補数)
#

export PATH=/usr/bin:/bin:/sbin:/wipeout:/:.
export PATTERNFIL=/tmp/pattern.dat

# 起動時のシステムメッセージを保存
dmesg > /var/run/dmesg.boot

# ツール起動
cd /wipeout
stuff -g > $PATTERNFIL
/bin/sh mainmenu.sh

# とりあえず対話シェルを動かしておく :-)
/bin/sh -i

exit 0

# EOF
