#! /bin/sh
#
#    crlf.sh -- 行末コードを LF (\n) から CRLF (\r\n) に
#
#	0.0: Feb.  5, 2007 by Dai ISHIJIMA
#	0.1: Feb. 18, 2007 by Dai ISHIJIMA
#

cr=`echo -e '\r'`

if sed 's/$/'"$cr"'/' ; then
	exit 0
else
	echo "crlf.sh: write failed"
	exit 1
fi

# EOF