#! /bin/sh
#
#   dummyread.sh -- キーボードから入力されている文字を読んで捨てる
#
#	0.0: Feb. 12, 2007 by Dai ISHIJIMA (for wpout08)
#

while read -t 0 keyin > /dev/null 1>&2 ; do
	case x"$DEBUG" in
		xY*)
			echo "|$keyin|"
			;;
	esac
done

exit 0

# EOF
